from __future__ import annotations

from collections.abc import Sequence

from rsb.models.base_model import BaseModel


class UpsertedFile(BaseModel):
    chunk_ids: Sequence[str]

    def __add__(self, other: UpsertedFile) -> UpsertedFile:
        return UpsertedFile(chunk_ids=list(self.chunk_ids) + list(other.chunk_ids))

    def __radd__(self, other: int | UpsertedFile) -> UpsertedFile:
        if other == 0:  # Handle sum()'s default start value
            return self
        elif isinstance(other, UpsertedFile):
            return UpsertedFile(chunk_ids=list(other.chunk_ids) + list(self.chunk_ids))
        else:
            return NotImplemented



import abc
from collections.abc import Mapping, Sequence
from typing import Any

from rsb.coroutines.run_sync import run_sync

from agentle.embeddings.models.embed_content import EmbedContent
from agentle.embeddings.models.embedding import Embedding
from agentle.embeddings.providers.embedding_provider import EmbeddingProvider
from agentle.generations.providers.base.generation_provider import GenerationProvider
from agentle.generations.tools.tool import Tool
from agentle.parsing.chunk import Chunk
from agentle.parsing.chunking.chunking_config import ChunkingConfig
from agentle.parsing.chunking.chunking_strategy import ChunkingStrategy
from agentle.parsing.parsed_file import ParsedFile
from agentle.vector_stores.upserted_file import UpsertedFile


class VectorStore(abc.ABC):
    default_collection_name: str
    embedding_provider: EmbeddingProvider
    generation_provider: GenerationProvider | None

    def __init__(
        self,
        *,
        default_collection_name: str = "agentle",
        embedding_provider: EmbeddingProvider,
        generation_provider: GenerationProvider | None,
    ) -> None:
        self.default_collection_name = default_collection_name
        self.embedding_provider = embedding_provider
        self.generation_provider = generation_provider

    async def find_related_content(
        self, query: str | Sequence[Embedding] | Sequence[float], *, k: int = 10
    ) -> Sequence[Chunk]: ...

    @abc.abstractmethod
    async def _find_related_content(
        self, query: Sequence[float], *, k: int = 10
    ) -> Sequence[Chunk]: ...

    async def upsert(
        self,
        points: Embedding | Sequence[float],
        *,
        timeout: float | None = None,
        collection_name: str | None = None,
        points_metadatas: Sequence[Mapping[str, Any]] | None = None,
    ) -> UpsertedFile:
        return run_sync(
            self.upsert_async,
            points=points,
            timeout=timeout,
            collection_name=collection_name,
            points_metadatas=points_metadatas,
        )

    @abc.abstractmethod
    async def upsert_async(
        self,
        points: Embedding | Sequence[float],
        *,
        timeout: float | None = None,
        collection_name: str | None = None,
        points_metadatas: Sequence[Mapping[str, Any]] | None = None,
    ) -> UpsertedFile:
        if len(points) == 0:
            return UpsertedFile(chunk_ids=[])

        if isinstance(points, Sequence):
            return await self._upsert_async(
                points=Embedding(value=points),
                timeout=timeout,
                collection_name=collection_name,
                points_metadatas=points_metadatas,
            )

        return await self._upsert_async(
            points=points,
            timeout=timeout,
            collection_name=collection_name,
            points_metadatas=points_metadatas,
        )

    @abc.abstractmethod
    async def _upsert_async(
        self,
        points: Embedding,
        *,
        timeout: float | None = None,
        collection_name: str | None = None,
        points_metadatas: Sequence[Mapping[str, Any]] | None = None,
    ) -> UpsertedFile: ...

    def upsert_file(
        self,
        file: ParsedFile,
        *,
        timeout: float | None = None,
        chunking_strategy: ChunkingStrategy,
        chunking_config: ChunkingConfig,
        collection_name: str | None,
    ) -> UpsertedFile:
        return run_sync(
            self.upsert_file_async,
            file=file,
            timeout=timeout,
            chunking_strategy=chunking_strategy,
            chunking_config=chunking_config,
            collection_name=collection_name,
        )

    async def upsert_file_async(
        self,
        file: ParsedFile,
        *,
        chunking_strategy: ChunkingStrategy,
        chunking_config: ChunkingConfig,
        collection_name: str | None,
    ) -> UpsertedFile:
        chunks: Sequence[Chunk] = await file.chunkify_async(
            strategy=chunking_strategy, config=chunking_config
        )

        embed_contents: Sequence[EmbedContent] = [
            await self.embedding_provider.generate_embeddings_async(
                c.text, metadata=c.metadata
            )
            for c in chunks
        ]

        upserted_chunks = [
            await self.upsert_async(
                points=e.embeddings,
                collection_name=collection_name,
                points_metadatas=[c.metadata for c in chunks],
            )
            for e in embed_contents
        ]

        return sum(upserted_chunks) or UpsertedFile(chunk_ids=[])

    def as_tool(self) -> Tool[Sequence[Chunk]]: ...  # TODO(arthur)



from collections.abc import Awaitable, Callable
from typing import Any, Optional

from agentle.embeddings.providers.embedding_provider import EmbeddingProvider
from agentle.generations.providers.base.generation_provider import GenerationProvider
from agentle.vector_stores.vector_store import VectorStore


class QdrantVectorStore(VectorStore):
    def __init__(
        self,
        *,
        default_collection_name: str = "agentle",
        embedding_provider: EmbeddingProvider,
        generation_provider: GenerationProvider | None,
        location: Optional[str] = None,
        url: Optional[str] = None,
        port: Optional[int] = 6333,
        grpc_port: int = 6334,
        prefer_grpc: bool = False,
        https: Optional[bool] = None,
        api_key: Optional[str] = None,
        prefix: Optional[str] = None,
        timeout: Optional[int] = None,
        host: Optional[str] = None,
        path: Optional[str] = None,
        force_disable_check_same_thread: bool = False,
        grpc_options: Optional[dict[str, Any]] = None,
        auth_token_provider: Optional[
            Callable[[], str] | Callable[[], Awaitable[str]]
        ] = None,
        cloud_inference: bool = False,
        local_inference_batch_size: Optional[int] = None,
        check_compatibility: bool = True,
        **kwargs: Any,
    ) -> None:
        from qdrant_client.async_qdrant_client import AsyncQdrantClient

        super().__init__(
            default_collection_name=default_collection_name,
            embedding_provider=embedding_provider,
            generation_provider=generation_provider,
        )

        self._client = AsyncQdrantClient(
            location=location,
            url=url,
            port=port,
            grpc_port=grpc_port,
            prefer_grpc=prefer_grpc,
            https=https,
            api_key=api_key,
            prefix=prefix,
            timeout=timeout,
            host=host,
            path=path,
            force_disable_check_same_thread=force_disable_check_same_thread,
            grpc_options=grpc_options,
            auth_token_provider=auth_token_provider,
            cloud_inference=cloud_inference,
            local_inference_batch_size=local_inference_batch_size,
            check_compatibility=check_compatibility,
            **kwargs,
        )



from typing import Any, Mapping
from rsb.models.base_model import BaseModel
from rsb.models.field import Field


class Chunk(BaseModel):
    text: str = Field(description="The text of the chunk.")
    metadata: Mapping[str, Any] = Field(default_factory=dict)
